@@ -1,6 +1,6 @@
 /*
- * static char *rcsid_client_types_h =
- *   "$Id$";
+ * static char *rcsid_common_client_types_h =
+ *   "$Id: client-types.h 12149 2009-08-20 02:11:28Z kbulgrien $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -24,6 +24,11 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
+/**
+ * @file common/client-types.h
+ *
+ */
+
 #ifndef CLIENT_TYPES_H
 #define CLIENT_TYPES_H
 #include <cconfig.h>
@@ -55,14 +60,6 @@
 #  include <fcntl.h>
 #endif
 
-#ifdef WIN32
-#  include <winsock2.h>
-#endif
-
-#ifdef HAVE_PTHREAD_H
-#  include <pthread.h>
-#endif
-
 /* Just some handy ones I like to use */
 #ifndef FALSE
 #define FALSE 0
@@ -110,15 +107,8 @@ typedef signed char       sint8;
 #define FMT64               "I64d"
 #define FMT64U              "I64u"
 
-/* mingw defines rand and srand only */
-#ifndef random
-#define random rand
-#endif
-#ifndef srandom
-#define srandom srand
 #endif
 
-#endif
 
 #ifndef SOL_TCP
 #define SOL_TCP IPPROTO_TCP
