@@ -1,9 +1,9 @@
 const char * const rcsid_common_client_c =
-    "$Id$";
+    "$Id: client.c 12959 2010-04-26 06:01:54Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2001,2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,24 +22,24 @@ const char * const rcsid_common_client_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
- /* Client interface main routine.
-  * this file sets up a few global variables, connects to the server,
-  * tells it what kind of pictures it wants, adds the client and enters
-  * the main dispatch loop
-  *
-  * the main event loop (event_loop()) checks the tcp socket for input and
-  * then polls for x events.  This should be fixed since you can just block
-  * on both filedescriptors.
-  *
-  * The DoClient function recieves a message (an ArgList), unpacks it, and
-  * in a slow for loop dispatches the command to the right function through
-  * the commands table.   ArgLists are essentially like RPC things, only
-  * they don't require going through RPCgen, and it's easy to get variable
-  * length lists.  They are just lists of longs, strings, characters, and
-  * byte arrays that can be converted to a machine independent format
+/**
+ * @file common/client.c
+ * Client interface main routine.  Sets up a few global variables, connects to
+ * the server, tells it what kind of pictures it wants, adds the client and
+ * enters the main dispatch loop.
+ *
+ * The main event loop (event_loop()) checks the TCP socket for input and then
+ * polls for x events.  This should be fixed since you can just block on both
+ * filedescriptors.
+ *
+ * The DoClient function receives a message (an ArgList), unpacks it, and in a
+ * slow for loop dispatches the command to the right function through the
+ * commands table.   ArgLists are essentially like RPC things, only they don't
+ * require going through RPCgen, and it's easy to get variable length lists.
+ * They are just lists of longs, strings, characters, and byte arrays that can
+ * be converted to a machine independent format
  */
 
-
 #include <client.h>
 #include <external.h>
 #include <errno.h>
@@ -56,7 +56,7 @@ char *server=SERVER;
 char *server=NULL;
 #endif
 
-char VERSION_INFO[256];
+char VERSION_INFO[MAX_BUF];
 
 char *client_libdir=NULL,*meta_server=META_SERVER;
 char *sound_server="cfsndserv";
@@ -66,13 +66,14 @@ int last_used_skills[MAX_SKILL+1];
 
 int meta_port=META_PORT, want_skill_exp=0,
     replyinfo_status=0, requestinfo_sent=0, replyinfo_last_face=0,
-    maxfd,metaserver_on=METASERVER, metaserver2_on=METASERVER2;
+    maxfd,metaserver_on=METASERVER, metaserver2_on=METASERVER2,
+    wantloginmethod=0, serverloginmethod=0;
 uint32	tick=0;
 
 uint16	exp_table_max=0;
 uint64	*exp_table=NULL;
 
-int command_inscribe = 0;
+NameMapping skill_mapping[MAX_SKILL], resist_mapping[NUM_RESISTS];
 
 Client_Player cpl;
 ClientSocket csocket;
@@ -121,7 +122,6 @@ struct CmdMapping commands[] = {
     { "tick", TickCmd, INT_ARRAY /* uint32 */},
 
 
-    { "sound", SoundCmd, MIXED /* int8, int8, int16, int8 */},
     { "sound2", Sound2Cmd, MIXED /* int8, int8, int8, int8, int8, int8, chars, int8, chars */},
     { "music", (CmdProc)MusicCmd, ASCII },
     { "anim", AnimCmd, SHORT_ARRAY},
@@ -135,56 +135,71 @@ struct CmdMapping commands[] = {
     { "version", (CmdProc)VersionCmd, ASCII },
     { "goodbye", (CmdProc)GoodbyeCmd, NODATA },
     { "setup", (CmdProc)SetupCmd, ASCII},
-    { "ExtendedInfoSet", (CmdProc)ExtendedInfoSetCmd, NODATA},
+    { "failure", (CmdProc)FailureCmd, ASCII},
+    { "accountplayers", (CmdProc)AccountPlayersCmd, ASCII},
 
     { "query", (CmdProc)handle_query, ASCII},
     { "replyinfo", ReplyInfoCmd, ASCII},
     { "ExtendedTextSet", (CmdProc)SinkCmd, NODATA},
+    { "ExtendedInfoSet", (CmdProc)SinkCmd, NODATA},
 
     { "pickup", PickupCmd, INT_ARRAY /* uint32 */},
 };
 
 #define NCOMMANDS ((int)(sizeof(commands)/sizeof(struct CmdMapping)))
 
+/**
+ * Basic little function that closes the connection to the server.
+ * it seems better to have it one palce here than the same
+ * logic sprinkled about in half a dozen locations in the code.
+ * also useful in that if this logic does change, just one place
+ * to update it.
+ */
+void close_server_connection()
+{
+#ifdef WIN32
+    closesocket(csocket.fd);
+#else
+    close(csocket.fd);
+#endif
+    csocket.fd=-1;
+}
+
 void DoClient(ClientSocket *csocket)
 {
     int i,len;
     unsigned char *data;
 
     while (1) {
-	i=SockList_ReadPacket(csocket->fd, &csocket->inbuf, MAXSOCKBUF-1);
-	if (i==-1) {
-	    /* Need to add some better logic here */
-#ifdef WIN32
-	    closesocket(csocket->fd);
-#else
-	    close(csocket->fd);
-#endif
-	    csocket->fd=-1;
-	    return;
-	}
-	if (i==0) return;   /* Don't have a full packet */
-	/* Terminate the buffer */
-	csocket->inbuf.buf[csocket->inbuf.len]='\0';
+        i=SockList_ReadPacket(csocket->fd, &csocket->inbuf, MAXSOCKBUF-1);
+        if (i==-1) {
+            /* Need to add some better logic here */
+            close_server_connection();
+            return;
+        }
+        if (i==0) return;   /* Don't have a full packet */
+
+        /* Terminate the buffer */
+        csocket->inbuf.buf[csocket->inbuf.len]='\0';
         data = csocket->inbuf.buf+2;
         while ((*data != ' ') && (*data != '\0')) ++data;
-	if (*data == ' ') {
-	    *data='\0';
-	    data++;
-	    len = csocket->inbuf.len - (data - csocket->inbuf.buf);
-	}
-	else len = 0;
-	for(i=0;i < NCOMMANDS;i++) {
-	    if (strcmp((char*)csocket->inbuf.buf+2,commands[i].cmdname)==0) {
-		    script_watch((char*)csocket->inbuf.buf+2,data,len,commands[i].cmdformat);
-		    commands[i].cmdproc(data,len);
-		    break;
-	    }
-	}
-	csocket->inbuf.len=0;
-	if (i == NCOMMANDS) {
-	    printf("Bad command from server (%s)\n",csocket->inbuf.buf+2);
-	}
+        if (*data == ' ') {
+            *data='\0';
+            data++;
+            len = csocket->inbuf.len - (data - csocket->inbuf.buf);
+        }
+        else len = 0;
+        for(i=0;i < NCOMMANDS;i++) {
+            if (strcmp((char*)csocket->inbuf.buf+2,commands[i].cmdname)==0) {
+                script_watch((char*)csocket->inbuf.buf+2,data,len,commands[i].cmdformat);
+                commands[i].cmdproc(data,len);
+                break;
+            }
+        }
+        csocket->inbuf.len=0;
+        if (i == NCOMMANDS) {
+            printf("Bad command from server (%s)\n",csocket->inbuf.buf+2);
+        }
     }
 }
 
@@ -331,156 +346,154 @@ int init_connection(char *host, int port)
     return fd;
 }
 
-/* This function negotiates/establishes the connection with the
- * server.
+/* This function negotiates/establishes the connection with the server.
  */
-
 void negotiate_connection(int sound)
 {
     int tries;
 
     SendVersion(csocket);
 
-    /* We need to get the version command fairly early on because
-     * we need to know if the server will support a request to use
-     * png images.  This isn't done the best, because if the server
-     * never sends the version command, we can loop here forever.
-     * However, if it doesn't send the version command, we have no idea
-     * what we are dealing with.
+    /* We need to get the version command fairly early on because we need to
+     * know if the server will support a request to use png images.  This
+     * isn't done the best, because if the server never sends the version
+     * command, we can loop here forever.  However, if it doesn't send the
+     * version command, we have no idea what we are dealing with.
      */
     tries=0;
     while (csocket.cs_version==0) {
-	DoClient(&csocket);
-	if (csocket.fd == -1) return;
+        DoClient(&csocket);
+        if (csocket.fd == -1) return;
 
-	usleep(10*1000);    /* 10 milliseconds */
-	tries++;
-	/* If we have't got a response in 10 seconds, bail out */
-	if (tries > 1000) {
-#ifdef WIN32
-	    closesocket(csocket.fd);
-#else
-	    close(csocket.fd);
-#endif
-	    csocket.fd=-1;
-	    return;
-	}
+        usleep(10*1000);    /* 10 milliseconds */
+        tries++;
+        /* If we have't got a response in 10 seconds, bail out */
+        if (tries > 1000) {
+            close_server_connection();
+            return;
+        }
     }
 
     if (csocket.sc_version<1023) {
-	LOG (LOG_WARNING,"common::negotiate_connection","Server does not support PNG images, yet that is all this client");
-	LOG (LOG_WARNING,"common::negotiate_connection","supports.  Either the server needs to be upgraded, or you need to");
-	LOG (LOG_WARNING,"common::negotiate_connection","downgrade your client.");
-	exit(1);
+        LOG (LOG_WARNING,"common::negotiate_connection","Server does not support PNG images, yet that is all this client");
+        LOG (LOG_WARNING,"common::negotiate_connection","supports.  Either the server needs to be upgraded, or you need to");
+        LOG (LOG_WARNING,"common::negotiate_connection","downgrade your client.");
+        exit(1);
     }
 
     /* If the user has specified a numeric face id, use it. If it is a string
      * like base, then that resolves to 0, so no real harm in that.
      */
     if (face_info.want_faceset) face_info.faceset = atoi(face_info.want_faceset);
+
+    /* For spellmon, try each acceptable level, but make sure the one the
+     * client prefers is last.
+     */
     cs_print_string(csocket.fd,
-	    "setup map2cmd 1 tick 1 sound %d%s sexp %d darkness %d newmapcmd 1 spellmon 1 faceset %d facecache %d exp64 1 itemcmd 2",
-	    sound>=0, (sound>=0) ? " sound2 3" : "", want_skill_exp,
-		    want_config[CONFIG_LIGHTING]?1:0, face_info.faceset,
-		    want_config[CONFIG_CACHE]);
+                    "setup map2cmd 1 tick 1 sound2 %d darkness %d spellmon 1 spellmon 2 "
+                    "faceset %d facecache %d want_pickup 1 loginmethod %d newmapcmd 1",
+                    (sound>=0) ? 3 : 0, want_config[CONFIG_LIGHTING]?1:0,
+                    face_info.faceset, want_config[CONFIG_CACHE], wantloginmethod);
 
     /* We can do this right now also - isn't any reason to wait */
+    cs_print_string(csocket.fd, "requestinfo skill_info");
     cs_print_string(csocket.fd,"requestinfo exp_table");
 
+    /* While these are only used for new login method, that
+     * should hopefully become standard fairly soon, and
+     * all of these are pretty small in any case, so don't
+     * add much to the cost.  In this way, we are more likely
+     * to have the information ready when we bring up the window.
+     */
+    cs_print_string(csocket.fd,"requestinfo motd");
+    cs_print_string(csocket.fd,"requestinfo news");
+    cs_print_string(csocket.fd,"requestinfo rules");
+
     use_config[CONFIG_MAPHEIGHT]=want_config[CONFIG_MAPHEIGHT];
     use_config[CONFIG_MAPWIDTH]=want_config[CONFIG_MAPWIDTH];
     mapdata_set_size(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
     if (use_config[CONFIG_MAPHEIGHT]!=11 || use_config[CONFIG_MAPWIDTH]!=11)
-	cs_print_string(csocket.fd,"setup mapsize %dx%d",use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
+        cs_print_string(csocket.fd,"setup mapsize %dx%d",use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 
     use_config[CONFIG_SMOOTH]=want_config[CONFIG_SMOOTH];
-    if (use_config[CONFIG_SMOOTH]){ /*or other mapextended infos*/
-        cs_print_string(csocket.fd,"setup extendedMapInfos 1");
-        /*will handle all special infos requested when setup answer this command*/
-    }
-    cs_print_string(csocket.fd,"setup extendedTextInfos 1");
-    cs_print_string(csocket.fd,"setup want_pickup 1");
-    cs_print_string(csocket.fd,"setup inscribe 1");
 
     /* If the server will answer the requestinfo for image_info and image_data,
      * send it and wait for the response.
      */
     if (csocket.sc_version >= 1027) {
-	/* last_start is -99.  This means the first face requested will
-	 * be 1 (not 0) - this is OK because 0 is defined as the blank
-	 * face.
-	 */
-	int last_end=0, last_start=-99;
+        /* last_start is -99.  This means the first face requested will be 1
+         * (not 0) - this is OK because 0 is defined as the blank face.
+         */
+        int last_end=0, last_start=-99;
 
-	cs_print_string(csocket.fd,"requestinfo image_info");
-	requestinfo_sent = RI_IMAGE_INFO;
-	replyinfo_status = 0;
-	replyinfo_last_face = 0;
+        cs_print_string(csocket.fd,"requestinfo image_info");
+        requestinfo_sent = RI_IMAGE_INFO;
+        replyinfo_status = 0;
+        replyinfo_last_face = 0;
 
-	do {
-	    DoClient(&csocket);
+        do {
+            DoClient(&csocket);
 
-	    /* it's rare, the connection can die while getting
-	     * this info.
-	     */
-	    if (csocket.fd == -1) return;
+            /* it's rare, the connection can die while getting this info.
+             */
+            if (csocket.fd == -1) return;
 
-	    if (use_config[CONFIG_DOWNLOAD]) {
-		/* we need to know how many faces to
-		 * be able to make the request intelligently.
-		 * So only do the following block if we have that info.
-		 * By setting the sent flag, we will never exit
-		 * this loop until that happens.
-		 */
-		requestinfo_sent |= RI_IMAGE_SUMS;
-		if (face_info.num_images != 0) {
-		    /* Sort of fake things out - if we have sent the
-		     * request for image sums but have not got them all answered
-		     * yet, we then clear the bit from the status
-		     * so we continue to loop.
-		     */
-		    if (last_end == face_info.num_images) {
-			/* Mark that we're all done */
-			if (replyinfo_last_face == last_end) {
-			    replyinfo_status |= RI_IMAGE_SUMS;
-			    image_update_download_status(face_info.num_images, face_info.num_images, face_info.num_images);
-			}
-		    } else {
-			/* If we are all caught up, request another
-			 * 100 sums.
-			 */
-			if (last_end == replyinfo_last_face) {
-			    last_start += 100;
-			    last_end += 100;
-			    if (last_end > face_info.num_images) last_end = face_info.num_images;
-			    cs_print_string(csocket.fd,"requestinfo image_sums %d %d", last_start, last_end);
-			    image_update_download_status(last_start, last_end, face_info.num_images);
-			}
-		    }
-		} /* Still have image_sums request to send */
-	    } /* endif download all faces */
+            if (use_config[CONFIG_DOWNLOAD]) {
+                /* we need to know how many faces to be able to make the
+                 * request intelligently.  So only do the following block if
+                 * we have that info.  By setting the sent flag, we will never
+                 * exit this loop until that happens.
+                 */
+                requestinfo_sent |= RI_IMAGE_SUMS;
+                if (face_info.num_images != 0) {
+                    /* Sort of fake things out - if we have sent the request
+                     * for image sums but have not got them all answered yet,
+                     * we then clear the bit from the status so we continue to
+                     * loop.
+                     */
+                    if (last_end == face_info.num_images) {
+                        /* Mark that we're all done */
+                        if (replyinfo_last_face == last_end) {
+                            replyinfo_status |= RI_IMAGE_SUMS;
+                            image_update_download_status(face_info.num_images, face_info.num_images, face_info.num_images);
+                        }
+                    } else {
+                        /* If we are all caught up, request another 100 sums.
+                         */
+                        if (last_end <= (replyinfo_last_face+100)) {
+                            last_start += 100;
+                            last_end += 100;
+                            if (last_end > face_info.num_images) last_end = face_info.num_images;
+                            cs_print_string(csocket.fd,"requestinfo image_sums %d %d", last_start, last_end);
+                            image_update_download_status(last_start, last_end, face_info.num_images);
+                        }
+                    }
+                } /* Still have image_sums request to send */
+            } /* endif download all faces */
 
-	    usleep(10*1000);    /* 10 milliseconds */
-	    /* Don't put in an upper time limit with tries like we did above - if the
-	     * player is downloading all the images, the time this takes could be
-	     * considerable.
-	     */
+            usleep(10*1000);    /* 10 milliseconds */
+            /* Don't put in an upper time limit with tries like we did above -
+             * if the player is downloading all the images, the time this
+             * takes could be considerable.
+             */
 
-	} while (replyinfo_status != requestinfo_sent);
+        } while (replyinfo_status != requestinfo_sent);
     }
     if (use_config[CONFIG_DOWNLOAD]) {
-	char buf[MAX_BUF];
+        char buf[MAX_BUF];
 
-	snprintf(buf, sizeof(buf), "Download of images complete.  Found %d locally, downloaded %d from server\n",
-		face_info.cache_hits, face_info.cache_misses);
-	draw_info(buf, NDI_GOLD);
+        snprintf(buf, sizeof(buf), "Download of images complete.  Found %d locally, downloaded %d from server\n",
+                 face_info.cache_hits, face_info.cache_misses);
+        draw_ext_info(NDI_GOLD, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_CONFIG, buf);
     }
 
-    /* This needs to get changed around - we really don't want to send
-     * the SendAddMe until we do all of our negotiation, which may include
-     * things like downloading all the images and whatnot - this is more an
-     * issue if the user is not using the default face set, as in that case,
-     * we might end up building images from the wrong set.
+    /* This needs to get changed around - we really don't want to send the
+     * SendAddMe until we do all of our negotiation, which may include things
+     * like downloading all the images and whatnot - this is more an issue if
+     * the user is not using the default face set, as in that case, we might
+     * end up building images from the wrong set.
+     * Only run this if not using new login method
      */
-    SendAddMe(csocket);
+    if (!serverloginmethod)
+        SendAddMe(csocket);
 }
