@@ -1,9 +1,9 @@
 const char * const rcsid_common_commands_c =
-    "$Id$";
+    "$Id: commands.c 12949 2010-04-25 07:07:43Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2001,2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,44 +22,44 @@ const char * const rcsid_common_commands_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-
-/* Handles commands received by the server.  This does not necessarily
- * handle all commands - some might be in other files (like init.c)
+/**
+ * @file common/commands.c
+ * Handles server->client commands; See player.c for client->server commands.
  *
- * This file handles commans from the server->client.  See player.c
- * for client->server commands.
+ * Not necessarily all commands are handled - some might be in other files
+ * (like init.c)
  *
- * this file contains most of the commands for the dispatch loop most of
- * the functions are self-explanatory, the pixmap/bitmap commands recieve
- * the picture, and display it.  The drawinfo command draws a string
- * in the info window, the stats command updates the local copy of the stats
- * and displays it. handle_query prompts the user for input.
- * send_reply sends off the reply for the input.
- * player command gets the player information.
- * MapScroll scrolls the map on the client by some amount
- * MapCmd displays the map either with layer packing or stack packing.
+ * This file contains most of the commands for the dispatch loop. Most of
+ * the functions are self-explanatory.
+ *
+ * pixmap/bitmap : receive the picture, and display it.
+ * drawinfo      : draws a string in the info window.
+ * stats         : updates the local copy of the stats and displays it.
+ * handle_query  : prompts the user for input.
+ * send_reply    : sends off the reply for the input.
+ * player        : gets the player information.
+ * MapScroll     : scrolls the map on the client by some amount.
+ * MapCmd        : displays the map with layer packing or stack packing.
  *   packing/unpacking is best understood by looking at the server code
  *   (server/ericserver.c)
- *   stack packing is easy, for every map entry that changed, we pack
- *   1 byte for the x/y location, 1 byte for the count, and 2 bytes per
- *   face in the stack.
- *   layer packing is harder, but I seem to remember more efficient:
- *   first we pack in a list of all map cells that changed and are now
- *   empty.  The end of this list is a 255, which is bigger that 121, the
- *   maximum packed map location.
- *   For each changed location we also pack in a list of all the faces and
- *   X/Y coordinates by layer, where the layer is the depth in the map.
- *   This essentially takes slices through the map rather than stacks.
+ *   stack packing: for every map entry that changed, we pack 1 byte for the
+ *   x/y location, 1 byte for the count, and 2 bytes per face in the stack.
+ *   layer packing is harder, but I seem to remember more efficient: first we
+ *   pack in a list of all map cells that changed and are now empty.  The end
+ *   of this list is a 255, which is bigger that 121, the maximum packed map
+ *   location.
+ *   For each changed location we also pack in a list of all the faces and X/Y
+ *   coordinates by layer, where the layer is the depth in the map.  This
+ *   essentially takes slices through the map rather than stacks.
  *   Then for each layer, (max is MAXMAPCELLFACES, a bad name) we start
- *   packing the layer into the message.  First we pack in a face, then
- *   for each place on the layer with the same face, we pack in the x/y
- *   location.  We mark the last x/y location with the high bit on
- *   (11*11 = 121 < 128).  We then continue on with the next face, which
- *   is why the code marks the faces as -1 if they are finished.  Finally
- *   we mark the last face in the layer again with the high bit, clearly
- *   limiting the total number of faces to 32767, the code comments it's
- *   16384, I'm not clear why, but the second bit may be used somewhere
- *   else as well.
+ *   packing the layer into the message.  First we pack in a face, then for
+ *   each place on the layer with the same face, we pack in the x/y location.
+ *   We mark the last x/y location with the high bit on (11*11 = 121 < 128).
+ *   We then continue on with the next face, which is why the code marks the
+ *   faces as -1 if they are finished.  Finally we mark the last face in the
+ *   layer again with the high bit, clearly limiting the total number of faces
+ *   to 32767, the code comments it's 16384, I'm not clear why, but the second
+ *   bit may be used somewhere else as well.
  *   The unpacking routines basically perform the opposite operations.
  */
 
@@ -71,7 +71,17 @@ int mapupdatesent = 0;
 
 #include "mapdata.h"
 
+char *news=NULL, *motd=NULL, *rules=NULL;
 
+int spellmon_level = 0;                 /**< Keeps track of what spellmon
+                                         *   command is supported by the
+                                         *   server. */
+
+/**
+ *
+ * @param data
+ * @param len
+ */
 static void get_exp_info(const unsigned char *data, int len) {
     int pos, level;
 
@@ -93,6 +103,11 @@ static void get_exp_info(const unsigned char *data, int len) {
     }
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 static void get_skill_info(char *data, int len) {
     char *cp, *nl, *sn;
     int val;
@@ -125,8 +140,12 @@ static void get_skill_info(char *data, int len) {
     } while (cp < data+len);
 }
 
-/* handles the response from a 'requestinfo' command.  This function doesn't
+/**
+ * Handles the response from a 'requestinfo' command.  This function doesn't
  * do much itself other than dispatch to other functions.
+ *
+ * @param buf
+ * @param len
  */
 void ReplyInfoCmd(uint8 *buf, int len) {
     uint8 *cp;
@@ -163,22 +182,38 @@ void ReplyInfoCmd(uint8 *buf, int len) {
         get_skill_info((char*)cp, len-i-1);   /* located in common/commands.c */
     } else if (!strcmp((char*)buf, "exp_table")) {
         get_exp_info(cp, len-i-1);   /* located in common/commands.c */
+    } else if (!strcmp((char*)buf, "motd")) {
+        if (motd) free((char*)motd);
+        motd = strdup(cp);
+        update_login_info(INFO_MOTD);
+    } else if (!strcmp((char*)buf, "news")) {
+        if (news) free((char*)news);
+        news = strdup(cp);
+        update_login_info(INFO_NEWS);
+    } else if (!strcmp((char*)buf, "rules")) {
+        if (rules) free((char*)rules);
+        rules = strdup(cp);
+        update_login_info(INFO_RULES);
     }
 }
 
-/* Received a response to a setup from the server.
- * This function is basically the same as the server side function - we
- * just do some different processing on the data.
+/**
+ * Received a response to a setup from the server.
+ * This function is basically the same as the server side function - we just
+ * do some different processing on the data.
+ *
+ * @param buf
+ * @param len
  */
 void SetupCmd(char *buf, int len) {
     int s;
     char *cmd, *param;
 
-    /* run through the cmds of setup
-     * syntax is setup <cmdname1> <parameter> <cmdname2> <parameter> ...
+    /* Process the setup commands.
+     * Syntax is setup <cmdname1> <parameter> <cmdname2> <parameter> ...
      *
-     * we send the status of the cmd back, or a FALSE is the cmd is the server unknown
-     * The client then must sort this out
+     * The server sends the status of the cmd back, or a FALSE if the cmd is
+     * unknown.  The client then must sort this out.
      */
 
     LOG(LOG_DEBUG, "common::SetupCmd", "%s", buf);
@@ -209,9 +244,9 @@ void SetupCmd(char *buf, int len) {
             s++;
         }
 
-        /* what we do with the returned data depends on what the server
-         * returns to us.  In some cases, we may fall back to other
-         * methods, just report on error, or try another setup command.
+        /* What is done with the returned data depends on what the server
+         * returns.  In some cases the client may fall back to other methods,
+         * report an error, or try another setup command.
          */
         if (!strcmp(cmd, "sound2")) {
             /* No parsing needed, but we don't want a warning about unknown
@@ -219,16 +254,15 @@ void SetupCmd(char *buf, int len) {
              */
         } else if (!strcmp(cmd, "sound")) {
             /* No, this should not be !strcmp()... */
-            if (strcmp(param, "FALSE")) {
-                cs_print_string(csocket.fd, "setsound %d", want_config[CONFIG_SOUND]);
-            }
         } else if (!strcmp(cmd, "mapsize")) {
             int x, y = 0;
             char *cp, tmpbuf[MAX_BUF];
 
             if (!strcasecmp(param, "false")) {
-                draw_info("Server only supports standard sized maps (11x11)", NDI_RED);
-                /* Do this because we may have been playing on a big server before */
+                draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER,
+                  "Server only supports standard sized maps (11x11)");
+                /* Do this because we may have been playing on a big server
+                 * before */
                 use_config[CONFIG_MAPWIDTH] = 11;
                 use_config[CONFIG_MAPHEIGHT] = 11;
                 mapdata_set_size(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
@@ -242,9 +276,10 @@ void SetupCmd(char *buf, int len) {
                     break;
                 }
             }
-            /* we wanted a size larger than the server supports.  Reduce our
-             * size to server maximum, and re-sent the setup command.
-             * Update our want sizes, and also tell the player what we are doing
+            /* A size larger than what the server supports was requested.
+             * Reduce the size to server maximum, and re-send the setup
+             * command.  Update our want sizes, and tell the player what is
+             * going on.
              */
             if (use_config[CONFIG_MAPWIDTH] > x || use_config[CONFIG_MAPHEIGHT] > y) {
                 if (use_config[CONFIG_MAPWIDTH] > x) use_config[CONFIG_MAPWIDTH] = x;
@@ -254,112 +289,83 @@ void SetupCmd(char *buf, int len) {
                                 "setup mapsize %dx%d", use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
                 snprintf(tmpbuf, sizeof(tmpbuf), "Server supports a max mapsize of %d x %d - requesting a %d x %d mapsize",
                     x, y, use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
-                draw_info(tmpbuf, NDI_RED);
+                draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER,
+                    tmpbuf);
             } else if (use_config[CONFIG_MAPWIDTH] == x && use_config[CONFIG_MAPHEIGHT] == y) {
                 mapdata_set_size(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
                 resize_map_window(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
             } else {
-                /* Our request was not bigger than what server supports, and
-                 * not the same size, so whats the problem?  Tell the user that
-                 * something is wrong.
+                /* The request was not bigger than what server supports, and
+                 * not the same size, so what is the problem?  Tell the user
+                 * that something is wrong.
                  */
                 snprintf(tmpbuf, sizeof(tmpbuf), "Unable to set mapsize on server - we wanted %d x %d, server returned %d x %d",
                     use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT], x, y);
-                draw_info(tmpbuf, NDI_RED);
+                draw_ext_info(
+                    NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER, tmpbuf);
             }
-        } else if (!strcmp(cmd, "sexp") || !strcmp(cmd, "darkness") ||
-            !strcmp(cmd, "newmapcmd") || !strcmp(cmd, "spellmon")) {
-            /* this really isn't an error or bug - in fact, it is expected if
-             * the user is playing on an older server.
+        } else if (!strcmp(cmd, "darkness")) {
+            /* Older servers might not support this setup command.
              */
             if (!strcmp(param, "FALSE")) {
-                LOG(LOG_WARNING, "common::SetupCmd", "Server returned FALSE on setup command %s", cmd);
-#if 0
-/* This should really be a callback to the gui if it needs to re-examine
- * the results here.
- */
-                if (!strcmp(cmd, "newmapcmd") && fog_of_war == TRUE) {
-                    fprintf(stderr, "**Warning: Fog of war is active but server does not support the newmap command\n");
-                }
-#endif
+                LOG(LOG_WARNING, "common::SetupCmd", "Server returned FALSE for setup command %s", cmd);
+            }
+        } else if (!strcmp(cmd, "spellmon")) {
+
+            /* Older servers might not support this setup command or all of
+             * the extensions.
+             *
+             * Spellmon 2 was added to the protocol in January 2010 to send an
+             * additional spell information string with casting requirements
+             * including required items, if the spell needs arguments passed
+             * (like text for rune of marking), etc.
+             *
+             * To use the new feature, "setup spellmon 1 spellmon 2" is sent,
+             * and if "spellmon 1 spellmon FALSE" is returned then the server
+             * doesn't accept 2 - sending spellmon 2 to a server that does not
+             * support it is not problematic, so the spellmon 1 command will
+             * still be handled correctly by the server.  If the server sends
+             * "spellmon 1 spellmon 2" then the extended mode is in effect.
+             *
+             * It is not particularly important for the player to know what
+             * level of command is accepted by the server.  The extra features
+             * will simply not be functionally available.
+             */
+            if (!strcmp(param, "FALSE")) {
+                LOG(LOG_INFO, "common::SetupCmd", "Server returned FALSE for a %s setup command", cmd);
+            } else {
+                spellmon_level = atoi(param);
             }
         } else if (!strcmp(cmd, "facecache")) {
-            if (!strcmp(param, "FALSE") && want_config[CONFIG_CACHE]) {
-                SendSetFaceMode(csocket, CF_FACE_CACHE);
-            } else {
                 use_config[CONFIG_CACHE] = atoi(param);
-            }
         } else if (!strcmp(cmd, "faceset")) {
             if (!strcmp(param, "FALSE")) {
-                draw_info("Server does not support other image sets, will use default", NDI_RED);
+                draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER,
+                    "Server does not support other image sets, will use default");
                 face_info.faceset = 0;
             }
         } else if (!strcmp(cmd, "map2cmd")) {
             if (!strcmp(param, "FALSE")) {
-                draw_info("Server does not support map2cmd!", NDI_RED);
-                draw_info("This server is too old to support this client!", NDI_RED);
-#ifdef WIN32
-                closesocket(csocket.fd);
-#else
-                close(csocket.fd);
-#endif
-                csocket.fd = -1;
-            }
-        } else if (!strcmp(cmd, "itemcmd")) {
-            /* don't really care - currently, the server will just
-             * fall back to the item1 transport mode.  If more item modes
-             * are used in the future, we should probably fall back one at
-             * a time or the like.
-             */
-        } else if (!strcmp(cmd, "exp64")) {
-            /* If this server does not support the new skill code,
-             * error out. If the server does support new exp system,
-             * send a request to get the mapping information.
-             */
-            if (!strcmp(param, "FALSE")) {
-                draw_info("Server does not support exp64!", NDI_RED);
-                draw_info("This server is too old to support this client!", NDI_RED);
-#ifdef WIN32
-                closesocket(csocket.fd);
-#else
-                close(csocket.fd);
-#endif
-                csocket.fd = -1;
-            } else {
-                cs_print_string(csocket.fd, "requestinfo skill_info");
-            }
-        } else if (!strcmp(cmd, "extendedMapInfos")) {
-            if (!strcmp(param, "FALSE")) {
-                use_config[CONFIG_SMOOTH] = 0;
-            } else {
-                /* Request all extended infos we want
-                 * Should regroup everything for easyness
-                 */
-                if (use_config[CONFIG_SMOOTH]) {
-                    cs_print_string(csocket.fd, "toggleextendedinfos smooth");
-                }
-            }
-        } else if (!strcmp(cmd, "extendedTextInfos")) {
-            if (strcmp(param, "FALSE")) { /* server didn't send FALSE*/
-                /* Server seems to accept extended text infos. Let's tell
-                 * it what extended text info we want
-                 */
-                char exttext[MAX_BUF];
-                TextManager *manager = firstTextManager;
-
-                while (manager) {
-                    snprintf(exttext, sizeof(exttext), "toggleextendedtext %d", manager->type);
-                    cs_print_string(csocket.fd, exttext);
-                    manager = manager->next;
-                }
+                draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER,
+                    "Server does not support map2cmd!");
+                draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SERVER,
+                    "This server is too old to support this client!");
+                close_server_connection();
             }
         } else if (!strcmp(cmd, "want_pickup")) {
-            /* Nothing to do specially, it's info pushed from server, not having it isn't that bad. */
-        } else if (!strcmp(cmd, "inscribe")) {
-            if (strcmp(param, "FALSE"))
-                command_inscribe = atoi(param);
-            else
-                command_inscribe = 0;
+            /* Nothing special to do as this is info pushed from server and
+             * not having it isn't that bad.
+             */
+        } else if (!strcmp(cmd, "loginmethod")) {
+            int method = atoi(param);
+
+            /* If the server supports new login, start the process.  Pass what
+             * version the server supports so client can do appropriate
+             * work
+             */
+            if (method) {
+                start_login(method);
+            }
         } else {
             LOG(LOG_INFO, "common::SetupCmd", "Got setup for a command we don't understand: %s %s",
                 cmd, param);
@@ -367,22 +373,13 @@ void SetupCmd(char *buf, int len) {
     }
 }
 
-void ExtendedInfoSetCmd(char *data, int len) {
-    (void)data; /* __UNUSED__ */
-    (void)len; /* __UNUSED__ */
-
-    /* Do nothing for now, perhaps later add some
-     * support to check what server knows.
-     */
-    /* commented, no waranty string data is null terminated
-       draw_info("ExtendedInfoSet returned from server: ", NDI_BLACK);
-       draw_info(data, NDI_BLACK);
-    */
-}
-
-/* Handles when the server says we can't be added.  In reality, we need to
- * close the connection and quit out, because the client is going to close
- * us down anyways.
+/**
+ * Handles when the server says we can't be added.  In reality, we need to
+ * close the connection and quit out, because the client is going to close us
+ * down anyways.
+ *
+ * @param data
+ * @param len
  */
 void AddMeFail(char *data, int len) {
     (void)data; /* __UNUSED__ */
@@ -392,17 +389,27 @@ void AddMeFail(char *data, int len) {
     return;
 }
 
-/* This is really a throwaway command - there really isn't any reason to
- * send addme_success commands.
+/**
+ * This is really a throwaway command - there really isn't any reason to send
+ * addme_success commands.
+ *
+ * @param data
+ * @param len
  */
 void AddMeSuccess(char *data, int len) {
     (void)data; /* __UNUSED__ */
     (void)len; /* __UNUSED__ */
 
+    hide_all_login_windows();
     LOG(LOG_INFO, "common::AddMeSuccess", "addme_success received.");
     return;
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void GoodbyeCmd(char *data, int len) {
     (void)data; /* __UNUSED__ */
     (void)len; /* __UNUSED__ */
@@ -418,6 +425,11 @@ void GoodbyeCmd(char *data, int len) {
 
 Animations animations[MAXANIM];
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void AnimCmd(unsigned char *data, int len) {
     short anum;
     int i, j;
@@ -453,10 +465,14 @@ void AnimCmd(unsigned char *data, int len) {
     LOG(LOG_DEBUG, "common::AnimCmd", "Received animation %d, %d faces", anum, animations[anum].num_animations);
 }
 
-/* This receives the smooth mapping from the server.  Because this
- * information is reference a lot, the smoothing face is stored
- * in the pixmap data - this makes access much faster than searching
- * an array of data for the face to use.
+/**
+ * Receives the smooth mapping from the server.  Because this information is
+ * reference a lot, the smoothing face is stored in the pixmap data - this
+ * makes access much faster than searching an array of data for the face to
+ * use.
+ *
+ * @param data
+ * @param len
  */
 void SmoothCmd(unsigned char *data, int len) {
     uint16 faceid;
@@ -472,6 +488,12 @@ void SmoothCmd(unsigned char *data, int len) {
     addsmooth(faceid, smoothing);
 }
 
+/**
+ * Draws a string in the info window.
+ *
+ * @param data
+ * @param len
+ */
 void DrawInfoCmd(char *data, int len) {
     int color = atoi(data);
     char *buf;
@@ -485,15 +507,16 @@ void DrawInfoCmd(char *data, int len) {
     } else {
         buf++;
     }
-    if (color != NDI_BLACK) {
-        draw_color_info(color, buf);
-    } else {
-        draw_info(buf, NDI_BLACK);
-    }
+    draw_ext_info(color, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_COMMAND, buf);
 }
 
 TextManager *firstTextManager = NULL;
 
+/**
+ *
+ * @param type
+ * @param callback
+ */
 void setTextManager(int type, ExtTextManager callback) {
     TextManager *current = firstTextManager;
 
@@ -511,6 +534,10 @@ void setTextManager(int type, ExtTextManager callback) {
     firstTextManager = current;
 }
 
+/**
+ *
+ * @param type
+ */
 static ExtTextManager getTextManager(int type) {
     TextManager *current = firstTextManager;
     while (current != NULL) {
@@ -522,7 +549,12 @@ static ExtTextManager getTextManager(int type) {
     return NULL;
 }
 
-/* We must extract color, type, subtype and dispatch to callback*/
+/**
+ * We must extract color, type, subtype and dispatch to callback
+ *
+ * @param data
+ * @param len
+ */
 void DrawExtInfoCmd(char *data, int len) {
     int color;
     int type, subtype;
@@ -568,18 +600,17 @@ void DrawExtInfoCmd(char *data, int len) {
     fnct(color, type, subtype, buf);
 }
 
-/* Maintain the last_used_skills LRU list for displaying the recently used
- * skills first. */
+/**
+ * Maintain the last_used_skills LRU list for displaying the recently used
+ * skills first.
+ *
+ * @param skill_id
+ */
 void use_skill(int skill_id)
 {
    int i = 0;
    int next;
    int prev = last_used_skills[0];
-   /*
-   char buf[100];
-   sprintf(buf, "use_skill(%d '%s')\n", skill_id, skill_names(skill_id));
-   draw_info(, NDI_BLUE);
-   */
 
    if(last_used_skills[0] == skill_id) return;
 
@@ -593,6 +624,12 @@ void use_skill(int skill_id)
    last_used_skills[0] = skill_id;
 }
 
+/**
+ * Updates the local copy of the stats and displays it.
+ *
+ * @param data
+ * @param len
+ */
 void StatsCmd(unsigned char *data, int len) {
     int i = 0, c, redraw = 0;
     sint64 last_exp;
@@ -714,6 +751,12 @@ void StatsCmd(unsigned char *data, int len) {
 #endif
 }
 
+/**
+ * Prompts the user for input.
+ *
+ * @param data
+ * @param len
+ */
 void handle_query(char *data, int len) {
     char *buf, *cp;
     uint8 flags = atoi(data);
@@ -740,7 +783,8 @@ void handle_query(char *data, int len) {
         cp = buf;
         while ((buf = strchr(buf, '\n')) != NULL) {
             *buf++ = '\0';
-            draw_info(cp, NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_QUERY, cp);
             cp = buf;
         }
         /* Yes/no - don't do anything with it now */
@@ -762,9 +806,11 @@ void handle_query(char *data, int len) {
     LOG(LOG_DEBUG, "common::handle_query", "Received query.  Input state now %d", cpl.input_state);
 }
 
-/* Sends a reply to the server.  text contains the null terminated
- * string of text to send.  This function basically just packs
- * the stuff up.
+/**
+ * Sends a reply to the server.
+ * This function basically just packs the stuff up.
+ *
+ * @param text contains the null terminated string of text to send.
  */
 void send_reply(const char *text) {
     cs_print_string(csocket.fd, "reply %s", text);
@@ -774,10 +820,13 @@ void send_reply(const char *text) {
     x_set_echo();
 }
 
-/* This function copies relevant data from the archetype to the
- * object.  Only copies data that was not set in the object
- * structure.
+/**
+ * Gets the player information.
+ * This function copies relevant data from the archetype to the object.  Only
+ * copies data that was not set in the object structure.
  *
+ * @param data
+ * @param len
  */
 void PlayerCmd(unsigned char *data, int len) {
     char name[MAX_BUF];
@@ -798,6 +847,10 @@ void PlayerCmd(unsigned char *data, int len) {
     new_player(tag, name, weight, face);
 }
 
+/**
+ *
+ * @param op
+ */
 void item_actions(item *op) {
     if (!op) {
         return;
@@ -812,9 +865,13 @@ void item_actions(item *op) {
     }
 }
 
-/* common_item_cmd parses the data send to us from the server.
- * revision is what item command the data came from - newer
- * ones have addition fields.
+/**
+ * Parses the data sent to us from the server.
+ * revision is what item command the data came from - newer ones have addition
+ * fields.
+ *
+ * @param data
+ * @param len
  */
 static void common_item_command(uint8 *data, int len) {
 
@@ -854,11 +911,21 @@ static void common_item_command(uint8 *data, int len) {
     }
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void Item2Cmd(unsigned char *data, int len) {
     common_item_command(data, len);
 }
 
-/* UpdateItemCmd updates some attributes of an item */
+/**
+ * Updates some attributes of an item
+ *
+ * @param data
+ * @param len
+ */
 void UpdateItemCmd(unsigned char *data, int len) {
     int weight, loc, tag, face, sendflags, flags, pos = 0, nlen, anim;
     uint32 nrof;
@@ -938,6 +1005,11 @@ void UpdateItemCmd(unsigned char *data, int len) {
     item_actions(locate_item(tag));
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void DeleteItem(unsigned char *data, int len) {
     int pos = 0, tag;
 
@@ -957,6 +1029,11 @@ void DeleteItem(unsigned char *data, int len) {
     }
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void DeleteInventory(unsigned char *data, int len) {
     int tag;
     item *op;
@@ -972,10 +1049,18 @@ void DeleteInventory(unsigned char *data, int len) {
     }
 }
 
-/******************************************************************************
- * Start of spell commands
- *****************************************************************************/
+/****************************************************************************/
 
+/**
+ * @defgroup SCSpellCommands Server->Client spell command functions.
+ * @{
+ */
+
+/**
+ *
+ * @param data
+ * @param len
+ */
 void AddspellCmd(unsigned char *data, int len) {
     uint8 nlen;
     uint16 mlen, pos = 0;
@@ -983,6 +1068,9 @@ void AddspellCmd(unsigned char *data, int len) {
 
     while (pos < len) {
         newspell = calloc(1, sizeof(Spell));
+
+        /* Get standard spell information (spellmon 1)
+         */
         newspell->tag = GetInt_String(data+pos); pos += 4;
         newspell->level = GetShort_String(data+pos); pos += 2;
         newspell->time = GetShort_String(data+pos); pos += 2;
@@ -998,9 +1086,32 @@ void AddspellCmd(unsigned char *data, int len) {
         mlen = GetShort_String(data+pos); pos += 2;
         strncpy(newspell->message, (char*)data+pos, mlen); pos += mlen;
         newspell->message[mlen] = '\0'; /* to ensure we are null terminated */
+
+        if (spellmon_level < 2) {
+
+          /* The server is not sending spellmon 2 extended information, so
+           * initialize the spell data fields as unused/empty.
+           */
+          newspell->usage = 0;
+          newspell->requirements[0] = '\0';
+
+        } else if (pos < len) {
+
+          /* The server is sending extended spell information (spellmon 2) so
+           * process it.
+           */
+          newspell->usage = GetChar_String(data+pos); pos += 1;
+          nlen = GetChar_String(data+pos); pos += 1;
+          strncpy(newspell->requirements, (char*) data+pos, nlen); pos += nlen;
+          newspell->requirements[nlen] = '\0'; /* Ensure null-termination */
+        }
+
+        /* Compute the derived spell information.
+         */
         newspell->skill = skill_names[newspell->skill_number-CS_STAT_SKILLINFO];
 
-        /* ok, we're done with putting in data, now to add to the player struct */
+        /* Add the spell to the player struct.
+         */
         if (!cpl.spelldata) {
             cpl.spelldata = newspell;
         } else {
@@ -1008,7 +1119,8 @@ void AddspellCmd(unsigned char *data, int len) {
                 ;
             tmp->next = newspell;
         }
-        /* now we'll check to see if we have more spells */
+        /* Check to see if there are more spells to add.
+         */
     }
     if (pos > len) {
         LOG(LOG_WARNING, "common::AddspellCmd", "Overread buffer: %d > %d", pos, len);
@@ -1016,6 +1128,11 @@ void AddspellCmd(unsigned char *data, int len) {
     cpl.spells_updated = 1;
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void UpdspellCmd(unsigned char *data, int len) {
     int flags, tag, pos = 0;
     Spell *tmp;
@@ -1048,6 +1165,11 @@ void UpdspellCmd(unsigned char *data, int len) {
     cpl.spells_updated = 1;
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void DeleteSpell(unsigned char *data, int len) {
     int tag;
     Spell *tmp, *target;
@@ -1086,10 +1208,22 @@ void DeleteSpell(unsigned char *data, int len) {
     cpl.spells_updated = 1;
 }
 
-/******************************************************************************
- * Start of map commands
- *****************************************************************************/
+/****************************************************************************/
 
+/**
+ * @} */ /* EndOf SCSpellCommands
+ */
+
+/**
+ * @defgroup SCMapCommands Server->Client map command functions.
+ * @{
+ */
+
+/**
+ *
+ * @param data
+ * @param len
+ */
 void NewmapCmd(unsigned char *data, int len) {
     (void)data; /* __UNUSED__ */
     (void)len; /* __UNUSED__ */
@@ -1113,6 +1247,11 @@ void NewmapCmd(unsigned char *data, int len) {
  */
 #define NUM_LAYERS (MAP1_LAYERS-1)
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void Map2Cmd(unsigned char *data, int len) {
     int mask, x, y, pos = 0, space_len, value;
     uint8 type;
@@ -1167,14 +1306,14 @@ void Map2Cmd(unsigned char *data, int len) {
             space_len = type>>5;
             type &= 0x1f;
             /* Clear the space */
-            if (type == 0) {
+            if (type == MAP2_TYPE_CLEAR) {
                 mapdata_clear_space(x, y);
                 continue;
-            } else if (type == 1) {
+            } else if (type == MAP2_TYPE_DARKNESS) {
                 value = data[pos++];
                 mapdata_set_darkness(x, y, value);
                 continue;
-            } else if (type >= 0x10 && type <= 0x1a) {
+            } else if (type >= MAP2_LAYER_START && type < MAP2_LAYER_START+MAXLAYERS) {
                 int layer, opt;
 
                 /* This is face information for a layer. */
@@ -1218,6 +1357,12 @@ void Map2Cmd(unsigned char *data, int len) {
     display_map_doneupdate(FALSE, FALSE);
 }
 
+/**
+ * Scrolls the map on the client by some amount.
+ *
+ * @param data
+ * @param len
+ */
 void map_scrollCmd(char *data, int len) {
     int dx, dy;
     char *buf;
@@ -1238,8 +1383,15 @@ void map_scrollCmd(char *data, int len) {
     display_map_doneupdate(FALSE, TRUE);
 }
 
-/* Extract smoothing infos from an extendedmapinfo packet part
- * data is located at the beginning of the smooth datas
+/**
+ * Extract smoothing infos from an extendedmapinfo packet part data is located
+ * at the beginning of the smooth datas
+ *
+ * @param data
+ * @param len
+ * @param x
+ * @param y
+ * @param layer
  */
 int ExtSmooth(unsigned char *data, int len, int x, int y, int layer) {
     static int dx[8] = { 0, 1, 1, 1, 0, -1, -1, -1, };
@@ -1269,11 +1421,14 @@ int ExtSmooth(unsigned char *data, int len, int x, int y, int layer) {
     return 1;/*Cause smooth infos only use 1 byte*/
 }
 
-/* Handle MapExtended command
- * Warning! if you add commands to extended, take
- * care that the 'layer' argument of main loop is
- * the opposite of the layer of the map.
- * so if you reference a layer, use NUM_LAYERS-layer
+/**
+ * Handle MapExtended command
+ * Warning! if you add commands to extended, take care that the 'layer'
+ * argument of main loop is the opposite of the layer of the map so if you
+ * reference a layer, use NUM_LAYERS-layer.
+ *
+ * @param data
+ * @param len
  */
 void MapExtendedCmd(unsigned char *data, int len) {
     int mask, x, y, pos = 0, layer;
@@ -1333,6 +1488,11 @@ void MapExtendedCmd(unsigned char *data, int len) {
     }
 }
 
+/**
+ *
+ * @param data
+ * @param len
+ */
 void MagicMapCmd(unsigned char *data, int len) {
     unsigned char *cp;
     int i;
@@ -1373,12 +1533,24 @@ void MagicMapCmd(unsigned char *data, int len) {
     draw_magic_map();
 }
 
+/**
+ * @} */ /* EndOf SCMapCommands
+ */
+
+/**
+ *
+ * @param data
+ * @param len
+ */
 void SinkCmd(unsigned char *data, int len) {
 }
 
-/* got a tick from the server.  We currently
- * don't care what tick number it is, but
- * just have the code in case at some time we do.
+/**
+ * Got a tick from the server.  We currently don't care what tick number it
+ * is, but just have the code in case at some time we do.
+ *
+ * @param data
+ * @param len
  */
 void TickCmd(uint8 *data, int len) {
 
@@ -1400,3 +1572,142 @@ void PickupCmd(uint8 *data, int len) {
     uint32 pickup = GetInt_String(data);
     client_pickup(pickup);
 }
+
+/**
+ * Handles a failure return from the server.
+ *
+ * @param data
+ * buffer sent by server.
+ * @param len
+ * length of data.
+ */
+void FailureCmd(char *buf, int len) {
+    char *cp;
+
+    /* The format of the buffer is 'command error message'.  We need to
+     * extract the failed command, and then pass in the error message
+     * to the appropriate handler.  So find the space, set it to null.
+     * in that way, buf is now just the failure command, and cp is the
+     * message.
+     */
+    cp = strchr(buf,' ');
+    if (!cp) return;
+
+    *cp = 0;
+    cp++;
+
+    if (!strcmp(buf,"accountlogin")) {
+        account_login_failure(cp);
+    }
+    else if (!strcmp(buf,"accountnew")) {
+        account_creation_failure(cp);
+    }
+    else if (!strcmp(buf,"accountaddplayer")) {
+        account_add_character_failure(cp);
+    }
+    else if (!strcmp(buf,"createplayer")) {
+        create_new_character_failure(cp);
+    }
+    else
+        /* This really is an error - if this happens it menas the server failed
+         * to process a request that the client made - the client should be able
+         * to handle failures for all request types it makes.  But this is also a problem
+         * in that it means that the server is waiting for a correct response, and
+         * if we do not display anything, the player is unlikely to know this.
+         */
+        LOG(LOG_ERROR, "common::FailureCmd", "Got a failure response we can not handle: %s:%s",
+            buf, cp);
+}
+
+/**
+ * This handles the accountplayers command
+ */
+void AccountPlayersCmd(char *buf, int len) {
+
+    int num_characters, level, pos, flen, faceno;
+    char name[MAX_BUF], class[MAX_BUF], race[MAX_BUF], 
+        face[MAX_BUF], party[MAX_BUF], map[MAX_BUF];
+
+
+    /* This is called first so it can clear out the existing
+     * data store.
+     */
+    choose_character_init();
+
+    level=0;
+    name[0]=0;
+    class[0]=0;
+    race[0]=0;
+    face[0]=0;
+    party[0]=0;
+    map[0]=0;
+    faceno=0;
+
+    /* We don't do anything with this right now */
+    num_characters=buf[0];
+    pos=1;
+    while (pos < len) {
+        flen = buf[pos];
+        /* flen == 0 is to note that we got end of character data */
+        if (flen == 0) {
+            update_character_choose(name, class, race, face, party, map, level, faceno);
+            /* blank all the values - it is no sure thing that the
+             * next character will fill all these in.
+             */
+            level=0;
+            name[0]=0;
+            class[0]=0;
+            race[0]=0;
+            face[0]=0;
+            party[0]=0;
+            map[0]=0;
+            faceno=0;
+            pos++;
+            continue;
+        }
+        pos++;
+        if ((pos +flen) > len || flen>=MAX_BUF) {
+            LOG(LOG_ERROR,"commands.c:AccountPlayerCmd", "data overran buffer");
+            return;
+        }
+        switch (buf[pos]) {
+        case ACL_NAME:
+            strncpy(name, buf + pos +1, flen-1);
+            name[flen-1] = 0;
+            break;
+
+        case ACL_CLASS:
+            strncpy(class, buf + pos +1, flen-1);
+            class[flen-1] = 0;
+            break;
+
+        case ACL_RACE:
+            strncpy(race, buf + pos +1, flen-1);
+            race[flen-1] = 0;
+            break;
+        
+        case ACL_FACE:
+            strncpy(face, buf + pos +1, flen-1);
+            face[flen-1] = 0;
+            break;
+
+        case ACL_PARTY:
+            strncpy(party, buf + pos +1, flen-1);
+            party[flen-1] = 0;
+            break;
+
+        case ACL_MAP:
+            strncpy(map, buf + pos +1, flen-1);
+            map[flen-1] = 0;
+            break;
+
+        case ACL_LEVEL:
+            level = GetShort_String(buf+pos+1);
+            break;
+        case ACL_FACE_NUM:
+            faceno = GetShort_String(buf+pos+1);
+            break;
+        }
+        pos += flen;
+    }
+}
