@@ -1,6 +1,6 @@
 /*
- * static char *rcsid_external_h =
- *   "$Id$";
+ * static char *rcsid_common_external_h =
+ *   "$Id: external.h 12949 2010-04-25 07:07:43Z mwedel $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -24,12 +24,12 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* This file contains external calls that the common area makes
- * callbacks to.
- * This was really a quick hack done to allow some seperation.
- * Really, these should be set via callbacks that the client
- * can make to the library.  Many of these probably should
- * never really be callbacks in any case, or be more general
+/**
+ * @file common/external.h
+ * Contains external calls that the common area makes callbacks to.  This was
+ * really a quick hack done to allow some separation.  Really, these should be
+ * set via callbacks that the client can make to the library.  Many of these
+ * probably should never really be callbacks in any case, or be more general.
  */
 
 /* Sound functions */
@@ -47,9 +47,8 @@ extern int display_mapscroll(int dx, int dy);
 extern void draw_magic_map(void);
 
 /* Info related functions */
-extern void draw_info(const char *str, int color);
-extern void draw_color_info(int colr, const char *buf);
 extern void draw_prompt(const char *str);
+extern void draw_ext_info(int orig_color, int type, int subtype, char *message);
 extern void x_set_echo(void);
 extern void set_scroll(const char *s);
 extern void set_autorepeat(const char *s);
@@ -89,5 +88,19 @@ extern void save_winpos(void);
 extern void save_defaults(void);
 extern void command_show(const char *params);
 extern void client_tick(uint32 tick);
-extern void cleanup_connection();
+extern void cleanup_connection(void);
 extern void client_pickup(uint32 pickup);
+
+/* Account Login Functions */
+extern void start_login(int method);
+extern void hide_all_login_windows();
+extern void account_login_failure(char *message);
+extern void account_creation_failure(char *message);
+extern void account_add_character_failure(char *message);
+extern void create_new_character_failure(char *message);
+extern void choose_character_init();
+extern void update_character_choose(const char *name, const char *class,
+                             const char *race, const char *face,
+                             const char *party, const char *map,
+                             int level, int faceno);
+extern void update_login_info(int type);
