@@ -1,9 +1,9 @@
 const char * const rcsid_common_init_c =
-    "$Id$";
+    "$Id: init.c 12875 2010-04-09 07:04:28Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2001,2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +22,10 @@ const char * const rcsid_common_init_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* This handles the initialization of the client.  This includes making
- * the I_IMAGE and I_ARCH commands.
+/**
+ * @file common/init.c
+ * Handles the initialization of the client.  This includes making the I_IMAGE
+ * and I_ARCH commands.
  */
 
 #include <client.h>
@@ -88,12 +90,6 @@ void SendAddMe(ClientSocket csock)
 }
 
 
-void SendSetFaceMode(ClientSocket csock,int mode)
-{
-    cs_print_string(csock.fd, "setfacemode %d", mode);
-}
-
-
 void init_client_vars(void)
 {
     int i;
@@ -285,15 +281,20 @@ void reset_client_vars(void)
     face_info.cache_misses=0;
     face_info.have_faceset_info=0;
     for (i=0; i<MAX_FACE_SETS; i++) {
-	FREE_AND_CLEAR(face_info.facesets[i].prefix);
-	FREE_AND_CLEAR(face_info.facesets[i].fullname);
-	face_info.facesets[i].fallback = 0;
-	FREE_AND_CLEAR(face_info.facesets[i].size);
-	FREE_AND_CLEAR(face_info.facesets[i].extension);
-	FREE_AND_CLEAR(face_info.facesets[i].comment);
+        FREE_AND_CLEAR(face_info.facesets[i].prefix);
+        FREE_AND_CLEAR(face_info.facesets[i].fullname);
+        face_info.facesets[i].fallback = 0;
+        FREE_AND_CLEAR(face_info.facesets[i].size);
+        FREE_AND_CLEAR(face_info.facesets[i].extension);
+        FREE_AND_CLEAR(face_info.facesets[i].comment);
     }
     reset_player_data();
     for (i=0; i<MAX_SKILL; i++)
         FREE_AND_CLEAR(skill_names[i]);
+    if (motd) FREE_AND_CLEAR(motd);
+    if (news) FREE_AND_CLEAR(news);
+    if (rules) FREE_AND_CLEAR(rules);
+
+    serverloginmethod=0;
 
 }
