@@ -1,9 +1,9 @@
 const char * const rcsid_common_metaserver_c =
-    "$Id$";
+    "$Id: metaserver.c 12946 2010-04-24 19:44:18Z cavesomething $";
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2001-2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@ const char * const rcsid_common_metaserver_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* This file deals with contact the metaserver, getting a list of hosts,
- * displaying/returning them to calling function, and then connecting
- * to the server when requested.
+/**
+ * @file common/metaserver.c
+ * Deals with contacting the metaserver, getting a list of hosts, displaying
+ * and returning them to calling function, and then connecting to the server
+ * when requested.
  */
 
 #ifndef WIN32
@@ -37,12 +39,12 @@ const char * const rcsid_common_metaserver_c =
 
 #include <ctype.h>
 #include <stdio.h>
-#include <stdlib.h>
 
 #include <client.h>
 #include <cconfig.h>
 #include <external.h>
 
+#include <pthread.h>
 #include <metaserver.h>
 
 #ifdef HAVE_CURL_CURL_H
@@ -56,6 +58,8 @@ Meta_Info *meta_servers = NULL;
 
 int meta_numservers = 0;
 
+int meta_sort(Meta_Info *m1, Meta_Info *m2) { return strcasecmp(m1->hostname, m2->hostname); }
+
 /* This checks the servers sc_version and cs_version to see
  * if they are compatible.
  * @parm entry
@@ -396,10 +400,12 @@ static int get_metaserver2_data(char *metaserver2) {
  */
 void *metaserver2_thread(void *junk)
 {
-    int metaserver_choice;
+    int metaserver_choice, tries=0;
 
     do {
 	metaserver_choice = random() % (sizeof(metaservers) / sizeof(char*));
+        tries++;
+        if (tries>5) break;
     } while (!get_metaserver2_data(metaservers[metaserver_choice]));
 
     pthread_mutex_lock(&ms2_info_mutex);
@@ -499,19 +505,19 @@ char *get_line_from_sock(char *s, size_t n, int fd) {
     }
 
     if (charsleft > MS_LARGE_BUF*4-3 && strchr(inbuf, '\n') == NULL) {
-        draw_info("Metaserver returned an overly long line.", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Metaserver returned an overly long line.");
         return NULL;
     }
 
     /* If there is no line in the buffer */
     while (charsleft == 0 || (cp = strchr(inbuf, '\n')) == NULL) {
-        FD_SET fdset;
+        FD_SET fdset = {1, fd};
         TIMEVAL tv = {3, 0}; /* 3 second timeout on reads */
         int nlen;
-        FD_ZERO(&fdset);
-        FD_SET(fd, &fdset);
         if (select(0, &fdset, NULL, NULL, &tv) == 0) {
-            draw_info("Metaserver timed out.", NDI_BLACK);
+            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+                "Metaserver timed out.");
             return NULL;
         }
 
@@ -590,7 +596,8 @@ void *metaserver1_thread(void *junk)
     }
     if (connect(fd, (struct sockaddr *)&insock, sizeof(insock)) == -1) {
         perror("Can't connect to metaserver");
-        draw_info("\nCan't connect to metaserver.", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "\nCan't connect to metaserver.");
 	pthread_mutex_lock(&ms2_info_mutex);
 	ms1_is_running=0;
 	pthread_mutex_unlock(&ms2_info_mutex);
@@ -721,7 +728,7 @@ void *metaserver1_thread(void *junk)
     ms1_is_running=0;
     pthread_mutex_unlock(&ms2_info_mutex);
     pthread_exit(NULL);
-    // never reached, just to make the compiler happy.
+    /* never reached, just to make the compiler happy. */
     return NULL;
 }
 
@@ -801,7 +808,7 @@ int metaserver_get_info(char *metaserver, int meta_port) {
     return 0;
 }
 
-/* show the metaservers to the player.  we use the draw_info to do
+/* Show the metaservers to the player.  We use draw_ext_info() to do
  * that, and also let the player know they can enter their own host name.
  */
 void metaserver_show(int show_selection) {
@@ -809,20 +816,24 @@ void metaserver_show(int show_selection) {
     char buf[256];
 
     if (cached_servers_num) {
-        draw_info("\nLast servers you connected to:\n", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "\nLast servers you connected to:\n");
         for (i = 0; i < cached_servers_num; i++) {
             snprintf(buf, sizeof(buf), "%2d) %-20.20s %-20.20s", i+1, cached_servers_name[i], cached_servers_ip[i]);
-            draw_info(buf, NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER, buf);
         }
-        draw_info(" ", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER, " ");
     }
 
     while(metaserver_check_status()) {
 	usleep(100);
     }
 
-    draw_info(" #)     Server        #     version   idle", NDI_BLACK);
-    draw_info("         Name      players           seconds", NDI_BLACK);
+    draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+        " #)     Server        #     version   idle");
+    draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+        "         Name      players           seconds");
     pthread_mutex_lock(&ms2_info_mutex);
 
     /* Re-sort the data - may get different data from ms1 and ms2, so
@@ -835,20 +846,26 @@ void metaserver_show(int show_selection) {
 		    i+1+cached_servers_num, meta_servers[i].hostname,
 		    meta_servers[i].num_players, meta_servers[i].version,
 		    meta_servers[i].idle_time);
-	    draw_info(buf, NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER, buf);
 	}
     }
     if (show_selection) {
         /* Show default/current server */
 	if (server) {
 	    snprintf(buf, sizeof(buf), "%2d)  %s (default)", meta_numservers+1+cached_servers_num, server);
-	    draw_info(buf, NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER, buf);
 	}
 
-        draw_info("Choose one of the entries above", NDI_BLACK);
-        draw_info("or type in a hostname/ip address", NDI_BLACK);
-        draw_info("Hit enter to re-update this list", NDI_BLACK);
-        draw_info("Enter 0 to exit the program.", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Choose one of the entries above");
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "or type in a hostname/ip address");
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Hit enter to re-update this list");
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Enter 0 to exit the program.");
     }
     pthread_mutex_unlock(&ms2_info_mutex);
 }
@@ -892,7 +909,9 @@ int metaserver_select(char *sel) {
         server_ip = sel;
     } else {
         if (num <= 0 || num > meta_numservers+cached_servers_num+1) {
-            draw_info("Invalid selection. Try again", NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+                    "Invalid selection. Try again");
             return 1;
         }
 
@@ -910,7 +929,8 @@ int metaserver_select(char *sel) {
     }
     pthread_mutex_unlock(&ms2_info_mutex);
     if (!server_name) {
-        draw_info("Bad selection. Try again", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Bad selection. Try again");
         return 1;
     }
 
@@ -927,14 +947,15 @@ int metaserver_select(char *sel) {
     }
 
     snprintf(buf, sizeof(buf), "Trying to connect to %s:%d", server_name, port);
-    draw_info(buf, NDI_BLACK);
+    draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER, buf);
 #ifdef MULTKEYS
     csocket.fd = init_connection(server_name, port);
 #else
     csocket.fd = init_connection(server_ip, port);
 #endif
     if (csocket.fd == -1) {
-        draw_info("Unable to connect to server.", NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+            "Unable to connect to server.");
         return 1;
     }
 
@@ -979,68 +1000,70 @@ int metaserver_select(char *sel) {
 
 #ifdef MS_STANDALONE
 /* This is here just to verify that the code seems to be working
- * properly
+ * properly, this tests both metaserver one and metaserver2
  * To use this code, compile as:
- *  gcc -o metaserver -I. -DMS_STANDALONE metaserver.c
- */
-
-int main(int argc, char *argv[])
-{
-    int i;
-
-    metaserver_get_info(META_SERVER, META_PORT);
-    for (i = 0; i < meta_numservers; i++) {
-        printf("%s:%d:%s:%d:%s:%s\n",
-            meta_servers[i].ip_addr,
-            meta_servers[i].idle_time,
-            meta_servers[i].hostname,
-            meta_servers[i].num_players,
-            meta_servers[i].version,
-            meta_servers[i].text_comment);
-    }
-}
-
-#endif
-
-#ifdef MS2_STANDALONE
-/* This is here just to verify that the code seems to be working
- * properly
- * To use this code, compile as:
- *  gcc -o metaserver -I. -DMS2_STANDALONE metaserver.c misc.o -lcurl -lpthread
+ *  gcc -o metaserver -I. -DMS_STANDALONE metaserver.c misc.o -lcurl -lpthread
+ *  if you only want to have support for one type of server then use either
+ *  -DMS_SA_NOTMS1 or -DMS_SA_NOTMS2 to disable the metaserver you don't want.
+ *  The list of servers goes to stdout, the headers for the tables, status messages etc, go to stderr.
  */
 
 /* Following lines are to cover external symbols not
  * defined - trying to bring in the files the are defined
  * in just causes more dependencies, etc.
  */
-void draw_info(const char *str, int color) { }
+void draw_ext_info(int orig_color, int type, int subtype, char *message) {}
 int init_connection(char *host, int port) {}
 
-int metaserver_on=1, meta_port=0;
-char *server=NULL, *meta_server;
+int metaserver2_on=1, metaserver_on=1;
+char *server=NULL;
 sint16 use_config[CONFIG_NUMS];
 ClientSocket csocket;
+char *meta_server=META_SERVER;
+int meta_port=META_PORT;
 
-int main(int argc, char *argv[])
-{
+void handle_ms_data(int msservernum) {
     int i;
+	fprintf(stderr,"Collecting data from metaserver %d.", msservernum);
+	while (metaserver_check_status()) {
+		fprintf(stderr,".");
+	    sleep(1);
+	}
+	fprintf(stderr, "\nIp Address:Idle Time:Hostname:Players:Version:Comment\n");
+	for (i = 0; i < meta_numservers; i++) {
+	    printf("%s:%d:%s:%d:%s:%s\n",
+			meta_servers[i].ip_addr,
+			meta_servers[i].idle_time,
+			meta_servers[i].hostname,
+			meta_servers[i].num_players,
+			meta_servers[i].version,
+			meta_servers[i].text_comment);
+	}
+	fprintf(stderr, "%d servers found\n", meta_numservers);
+}
+
+int main(int argc, char *argv[]) {
+
+#ifdef MS_SA_NOTMS2
+	metaserver2_on=0;
+#endif
+#ifdef MS_SA_NOTMS1
+	metaserver_on=0;
+#endif
 
     init_metaserver();
-    metaserver2_get_info();
-    fprintf(stderr,"Collecting data.");
-    while (metaserver2_check_status()) {
-	fprintf(stderr,".");
-	sleep(1);
+    if(metaserver2_on) {
+    	metaserver2_get_info();
+    	handle_ms_data(2);
     }
-    fprintf(stderr,"\n");
-    for (i = 0; i < meta_numservers; i++) {
-        printf("%s:%d:%s:%d:%s:%s\n",
-            meta_servers[i].ip_addr,
-            meta_servers[i].idle_time,
-            meta_servers[i].hostname,
-            meta_servers[i].num_players,
-            meta_servers[i].version,
-            meta_servers[i].text_comment);
+    /* both metaservers use the same array to store the servers in, so we'll
+     * reset it here in order to get the results from the other metaserver. */
+    free(meta_servers);
+    meta_servers=NULL;
+    meta_numservers = 0;
+    if(metaserver_on) {
+    	metaserver1_get_info();
+    	handle_ms_data(1);
     }
 }
 
