@@ -1,5 +1,5 @@
 const char * const rcsid_common_newsocket_c =
-    "$Id$";
+    "$Id: newsocket.c 12881 2010-04-12 06:14:36Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -22,18 +22,19 @@ const char * const rcsid_common_newsocket_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* Made this either client or server specific for 0.95.2 release - getting
- * too complicated to keep them the same, and the common code is pretty much
+/**
+ * @file common/newsocket.c
+ * Made this either client or server specific for 0.95.2 release - getting too
+ * complicated to keep them the same, and the common code is pretty much
  * frozen now.
  */
 
-
 #include <stdio.h>
 #include <stdarg.h>
 #include <errno.h>
 
 #include <client.h>
-#include <newclient.h>
+#include <shared/newclient.h>
 #include <script.h>
 
 /* The LOG function is normally part of the libcross.  If client compile,
@@ -176,79 +177,89 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
 
     /* We already have a partial packet */
     if (sl->len<2) {
-	do {
 #ifndef WIN32
-	    stat=read(fd, sl->buf + sl->len, 2-sl->len);
-	} while ((stat==-1) && (errno==EINTR));
+        do {
+            stat=read(fd, sl->buf + sl->len, 2-sl->len);
+        } while ((stat==-1) && (errno==EINTR));
 #else
-	    stat=recv(fd, sl->buf + sl->len, 2-sl->len, 0);
-	} while ((stat==-1) && (WSAGetLastError()==EINTR));
+        do {
+            stat=recv(fd, sl->buf + sl->len, 2-sl->len, 0);
+        } while ((stat==-1) && (WSAGetLastError()==EINTR));
 #endif
-	if (stat<0) {
-	    /* In non blocking mode, EAGAIN is set when there is no
-	     * data available.
-	     */
+
+        if (stat<0) {
+            /* In non blocking mode, EAGAIN is set when there is no
+             * data available.
+             */
 #ifndef WIN32
-	    if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
+            if (errno!=EAGAIN && errno!=EWOULDBLOCK)
 #else
-	    if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK) {
+            if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK)
 #endif
-		perror("ReadPacket got an error.");
-		LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning -1",errno);
-		return -1;
-	    }
-	    return 0;	/*Error */
-	}
-	if (stat==0) return -1;
-	sl->len += stat;
+            {
+                perror("ReadPacket got an error.");
+                    LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning -1",errno);
+                    return -1;
+            }
+            return 0;	/*Error */
+        }
+        if (stat==0) return -1;
+
+        sl->len += stat;
 #ifdef CS_LOGSTATS
-	cst_tot.ibytes += stat;
-	cst_lst.ibytes += stat;
+        cst_tot.ibytes += stat;
+        cst_lst.ibytes += stat;
 #endif
-	if (stat<2) return 0;	/* Still don't have a full packet */
-	readsome=1;
+        if (stat<2) return 0;	/* Still don't have a full packet */
+        readsome=1;
     }
+
     /* Figure out how much more data we need to read.  Add 2 from the
      * end of this - size header information is not included.
      */
     toread = 2+(sl->buf[0] << 8) + sl->buf[1] - sl->len;
     if ((toread + sl->len) > len) {
-	LOG(llevError,"SockList_ReadPacket","Want to read more bytes than will fit in buffer.\n");
-	/* return error so the socket is closed */
-	return -1;
+        LOG(llevError,"SockList_ReadPacket","Want to read more bytes than will fit in buffer.\n");
+        /* return error so the socket is closed */
+        return -1;
     }
     do {
-	do {
 #ifndef WIN32
-	    stat = read(fd, sl->buf+ sl->len, toread);
-	} while ((stat<0) && (errno==EINTR));
+        do {
+            stat = read(fd, sl->buf+ sl->len, toread);
+        } while ((stat<0) && (errno==EINTR));
 #else
-	    stat = recv(fd, sl->buf+ sl->len, toread, 0);
-	} while ((stat<0) && (WSAGetLastError()==EINTR));
+        do {
+            stat = recv(fd, sl->buf+ sl->len, toread, 0);
+        } while ((stat<0) && (WSAGetLastError()==EINTR));
 #endif
-	if (stat<0) {
+        if (stat<0) {
+
 #ifndef WIN32
-	    if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
+            if (errno!=EAGAIN && errno!=EWOULDBLOCK)
 #else
-	    if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK) {
+            if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK)
 #endif
-		perror("ReadPacket got an error.");
-		LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning 0",errno);
-	    }
-	    return 0;	/*Error */
-	}
-	if (stat==0) return -1;
-	sl->len += stat;
+                {
+                    perror("ReadPacket got an error.");
+                    LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning 0",errno);
+                }
+                return 0;	/*Error */
+        }
+        if (stat==0) return -1;
+        sl->len += stat;
+
 #ifdef CS_LOGSTATS
-	cst_tot.ibytes += stat;
-	cst_lst.ibytes += stat;
+        cst_tot.ibytes += stat;
+        cst_lst.ibytes += stat;
 #endif
-	toread -= stat;
-	if (toread==0) return 1;
-	if (toread < 0) {
-	    LOG(llevError,"SockList_ReadPacket","SockList_ReadPacket: Read more bytes than desired.");
-	    return 1;
-	}
+        toread -= stat;
+        if (toread==0) return 1;
+
+        if (toread < 0) {
+            LOG(llevError,"SockList_ReadPacket","SockList_ReadPacket: Read more bytes than desired.");
+            return 1;
+        }
     } while (toread>0);
     return 0;
 }
