@@ -1,7 +1,7 @@
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2005,2006 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2005-2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,22 +20,22 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/*
+/**
+ * @file common/p_cmd.c
  * Contains a lot about the commands typed into the client.
  */
 
 #ifndef CPROTO
-/* use declartions from p_cmd.h instead of doing make proto on this file */
+/* use declarations from p_cmd.h instead of doing make proto on this file */
 
 #include <client.h>
 #include <external.h>
 #include <script.h>
 #include <p_cmd.h>
 
-/*
- *
- * Help commands.
- *
+/**
+ * @defgroup PCmdHelpCommands Common client player commands.
+ * @{
  */
 
 /* TODO This should really be under /help commands or something... */
@@ -59,15 +59,18 @@ name name name ...
 #ifdef HELP_USE_COLOR
 #error Oops, need to put them back.
 #else
-#define H1(a) draw_info(a, NDI_BLACK)
-#define H2(a) draw_info(a, NDI_BLACK)
-#define LINE(a) draw_info(a, NDI_BLACK)
+#define H1(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
+#define H2(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
+#define LINE(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
 #endif
 
 #define assumed_wrap get_info_width()
 
-
 /* TODO Help topics other than commands? Refer to other documents? */
+
+/**
+ *
+ */
 static void do_clienthelp_list(void) {
     ConsoleCommand ** commands_array;
     ConsoleCommand * commands_copy;
@@ -156,7 +159,10 @@ static void do_clienthelp_list(void) {
 #endif
 }
 
-
+/**
+ *
+ * @param cc
+ */
 static void show_help(const ConsoleCommand * cc) {
     {
         char buf[MAX_BUF];
@@ -178,7 +184,8 @@ static void show_help(const ConsoleCommand * cc) {
         long_help = cc->helpfunc();
 
         if (long_help != NULL) {
-            /* For a test, let's watch draw_info() choke on newlines. */
+            /* For a test, let's watch draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT,
+             * MSG_TYPE_CLIENT_NOTICE, ) choke on newlines. */
             /* TODO C line wrapping (get_info_width()), argh. Or move it to UI? */
             LINE(long_help);
         } else {
@@ -189,6 +196,10 @@ static void show_help(const ConsoleCommand * cc) {
     }
 }
 
+/**
+ *
+ * @param arg
+ */
 static void do_clienthelp(const char * arg) {
     const ConsoleCommand * cc;
 
@@ -202,7 +213,7 @@ static void do_clienthelp(const char * arg) {
     if (cc == NULL) {
         char buf[MAX_BUF];
         snprintf(buf, MAX_BUF - 1, "clienthelp: Unknown command %s.", arg);
-        draw_info(buf, NDI_BLACK);
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, buf);
         return;
     }
 
@@ -210,6 +221,9 @@ static void do_clienthelp(const char * arg) {
 
 }
 
+/**
+ *
+ */
 static const char * help_clienthelp(void) {
     return
         "Syntax:\n"
@@ -226,6 +240,10 @@ static const char * help_clienthelp(void) {
         "See also: serverhelp, help.";
 }
 
+/**
+ *
+ * @param arg
+ */
 static void do_serverhelp(const char * arg) {
 
     if (arg) {
@@ -238,6 +256,9 @@ static void do_serverhelp(const char * arg) {
     }
 }
 
+/**
+ *
+ */
 static const char * help_serverhelp(void) {
     return
         "Syntax:\n"
@@ -253,7 +274,10 @@ static const char * help_serverhelp(void) {
         "See also: clienthelp, help.";
 }
 
-
+/**
+ *
+ * @param cpnext
+ */
 static void command_help(const char *cpnext) {
     if (cpnext) {
         const ConsoleCommand * cc;
@@ -278,6 +302,9 @@ static void command_help(const char *cpnext) {
     }
 }
 
+/**
+ *
+ */
 static const char * help_help(void) {
     return
         "Syntax:\n"
@@ -296,17 +323,23 @@ static const char * help_help(void) {
         "See also: clienthelp, serverhelp.";
 }
 
-
-/*
- *
- * Other commands.
- *
+/**
+ * @} */ /* EndOf PCmdHelpCommands
  */
 
+/*
+ * Other commands.
+ */
+
+/**
+ *
+ * @param cpnext
+ */
 static void set_command_window(const char *cpnext)
 {
     if (!cpnext) {
-	draw_info("cwindow command requires a number parameter", NDI_BLACK);
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "cwindow command requires a number parameter");
     } else {
 	want_config[CONFIG_CWINDOW] = atoi(cpnext);
 	if (want_config[CONFIG_CWINDOW]<1 || want_config[CONFIG_CWINDOW]>127)
@@ -316,22 +349,29 @@ static void set_command_window(const char *cpnext)
     }
 }
 
-static void command_foodbep(const char *cpnext)
+/**
+ *
+ * @param cpnext
+ */
+static void command_foodbeep(const char *cpnext)
 {
    (void)cpnext; /* __UNUSED__ */
     if (want_config[CONFIG_FOODBEEP]) {
 	want_config[CONFIG_FOODBEEP]=0;
-	draw_info("Warning bell when low on food disabled", NDI_BLACK);
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "Warning bell when low on food disabled");
     } else {
 	want_config[CONFIG_FOODBEEP]=1;
-	draw_info("Warning bell when low on food enabled", NDI_BLACK);
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "Warning bell when low on food enabled");
     }
     use_config[CONFIG_FOODBEEP] = want_config[CONFIG_FOODBEEP];
 }
 
-
-
-
+/**
+ *
+ * @param cat
+ */
 const char * get_category_name(CommCat cat) {
     const char * cat_name;
 
@@ -349,7 +389,6 @@ const char * get_category_name(CommCat cat) {
     return cat_name;
 }
 
-
 /*
  * Command table.
  *
@@ -357,16 +396,25 @@ const char * get_category_name(CommCat cat) {
  */
 
 /* "Typecasters" (and some forwards) */
+
+/**
+ *
+ * @param ignored
+ */
 static void do_script_list(const char * ignored) { script_list(); }
+
+/**
+ *
+ * @param ignored
+ */
 static void do_clearinfo(const char * ignored) { menu_clear(); }
 
+/**
+ *
+ * @param ignored
+ */
 static void do_disconnect(const char * ignored) {
-#ifdef WIN32
-        closesocket(csocket.fd);
-#else
-        close(csocket.fd);
-#endif
-        csocket.fd=-1;
+    close_server_connection();
 
 	/* the gtk clients need to do some cleanup logic - otherwise,
 	 * they start hogging CPU.
@@ -379,14 +427,24 @@ static void do_disconnect(const char * ignored) {
 #ifndef DMALLOC_VERIFY_NOERROR
   #define DMALLOC_VERIFY_NOERROR  1
 #endif
+/**
+ *
+ * @param ignored
+ */
 static void do_dmalloc(const char * ignored) {
         if (dmalloc_verify(NULL)==DMALLOC_VERIFY_NOERROR)
-            draw_info("Heap checks out OK", NDI_BLACK);
+            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+                "Heap checks out OK");
         else
-            draw_info("Heap corruption detected", NDI_RED);
+            draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_ERROR,
+                "Heap corruption detected");
 }
 #endif
 
+/**
+ *
+ * @param ignored
+ */
 static void do_inv(const char * ignored) { print_inventory (cpl.ob); }
 
 static void do_magicmap(const char * ignored) {
@@ -394,36 +452,77 @@ static void do_magicmap(const char * ignored) {
         draw_magic_map();
 }
 
+/**
+ *
+ * @param ignored
+ */
 static void do_metaserver(const char * ignored) {
         if (!metaserver_get_info(meta_server, meta_port))
             metaserver_show(FALSE);
         else
-            draw_info("Unable to get metaserver information.", NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
+                    "Unable to get metaserver information.");
 }
 
+/**
+ *
+ * @param ignored
+ */
 static void do_savedefaults(const char * ignored) { save_defaults(); }
 
+/**
+ *
+ * @param ignored
+ */
 static void do_savewinpos(const char * ignored) { save_winpos(); }
 
+/**
+ *
+ * @param used
+ */
 static void do_take(const char * used) { command_take("take", used); /* I dunno why they want it. */ }
 
+/**
+ *
+ * @param ignored
+ */
 static void do_num_free_items(const char * ignored) {
     LOG(LOG_INFO,"common::extended_command","num_free_items=%d", num_free_items());
 }
 
+/**
+ *
+ * @param arg
+ */
 static void do_clienthelp(const char * arg); /* Forward. */
 
 /* Help "typecasters". */
 #include "../help/chelp.h"
 
+/**
+ *
+ */
 static const char * help_bind(void) { return HELP_BIND_LONG; }
 
+/**
+ *
+ */
 static const char * help_unbind(void) { return HELP_UNBIND_LONG; }
 
+/**
+ *
+ */
 static const char * help_magicmap(void) { return HELP_MAGICMAP_LONG; }
 
+/**
+ *
+ */
 static const char * help_inv(void) { return HELP_INV_LONG; }
 
+/**
+ *
+ */
 static const char * help_cwindow(void) {
     return
         "Syntax:\n"
@@ -437,6 +536,9 @@ static const char * help_cwindow(void) {
 	"(What does this mean, 'put a lid on it'?) TODO";
 }
 
+/**
+ *
+ */
 static const char * help_script(void) {
     return
         "Syntax:\n"
@@ -448,6 +550,9 @@ static const char * help_script(void) {
 	"See Documentation/Script.html";
 }
 
+/**
+ *
+ */
 static const char * help_scripttell(void) {
     return
         "Syntax:\n"
@@ -458,6 +563,10 @@ static const char * help_scripttell(void) {
 }
 
 /* Toolkit-dependent. */
+
+/**
+ *
+ */
 static const char * help_savewinpos(void) {
     return
         "Syntax:\n"
@@ -467,6 +576,9 @@ static const char * help_savewinpos(void) {
         "save window positions - split windows mode only.";
 }
 
+/**
+ *
+ */
 static const char * help_metaserver(void) {
     /* TODO Add command_escape() where appropriate. On the other
     hand, that can lead to a meaningless syntax-display API.*/
@@ -484,6 +596,9 @@ static const char * help_metaserver(void) {
 	"Warning: This command may freeze the client until it gets the list.";
 }
 
+/**
+ *
+ */
 static const char * help_scriptkill(void) {
     return
         "Syntax:\n"
@@ -494,6 +609,9 @@ static const char * help_scriptkill(void) {
 	"(Not guaranteed to work?)";
 }
 
+/**
+ *
+ */
 static const char * help_showweight(void) {
     return
         "Syntax:\n"
@@ -510,25 +628,44 @@ static const char * help_showweight(void) {
 }
 
 /*
-*	draw_info("Information Commands", NDI_NAVY);*
-	draw_info(" inv         - *recursively* print your", NDI_BLACK);
-	draw_info("               inventory - includes containers.", NDI_BLACK);
-	draw_info(" mapredraw, showinfo, take", NDI_BLACK);
-	draw_info(" help        - show this message", NDI_BLACK);
-	draw_info(" help <command> - get more information on a", NDI_BLACK);
-	draw_info("                command (Server command only?)", NDI_BLACK);
-	draw_info(" showicon    - draw status icons in", NDI_BLACK);
-	draw_info("               inventory window", NDI_BLACK);
-	draw_info(" showweight  - show weight in inventory", NDI_BLACK);
-	draw_info("               look windows", NDI_BLACK);
-	draw_info("Scripting Commands", NDI_NAVY);
-	draw_info("Client Side Debugging Commands", NDI_NAVY);
+*	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "Information Commands");*
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " inv         - *recursively* print your");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "               inventory - includes containers.");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " mapredraw, showinfo, take");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " help        - show this message");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " help <command> - get more information on a");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "                command (Server command only?)");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " showicon    - draw status icons in");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "               inventory window");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " showweight  - show weight in inventory");
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "               look windows");
+	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "Scripting Commands");
+	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            "Client Side Debugging Commands");
 #ifdef HAVE_DMALLOC_H
-	draw_info(" dmalloc     - Check heap?", NDI_BLACK);
+	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+            " dmalloc     - Check heap?");
 #endif
 */
 
 /* Forward. */
+
+/**
+ *
+ * @param currently_ignored
+ */
 static void do_clienthelp(const char * currently_ignored);
 
 /* TODO Wrap these? Um. */
@@ -616,7 +753,7 @@ static ConsoleCommand CommonCommands[] = {
 
     {
         "foodbeep", COMM_CAT_SETUP,
-        command_foodbep, NULL,
+        command_foodbeep, NULL,
         "toggle audible low on food warning"
 
     },
@@ -724,10 +861,18 @@ extern const int ToolkitCommandsSize;
 
 int num_commands;
 
+/**
+ *
+ */
 int get_num_commands(void) { return num_commands; }
 
 static ConsoleCommand ** name_sorted_commands;
 
+/**
+ *
+ * @param a_
+ * @param b_
+ */
 static int sort_by_name(const void * a_, const void * b_)
 {
     ConsoleCommand * a = *((ConsoleCommand **)a_);
@@ -739,6 +884,12 @@ static int sort_by_name(const void * a_, const void * b_)
 static ConsoleCommand ** cat_sorted_commands;
 
 /* Sort by category, then by name. */
+
+/**
+ *
+ * @param a_
+ * @param b_
+ */
 static int sort_by_category(const void *a_, const void *b_)
 {
     /* Typecasts, so it goes. */
@@ -752,6 +903,9 @@ static int sort_by_category(const void *a_, const void *b_)
     return a->cat - b->cat;
 }
 
+/**
+ *
+ */
 void init_commands(void) {
     int i;
 
@@ -799,6 +953,10 @@ void init_commands(void) {
 #define tolower(C)      (((C) >= 'A' && (C) <= 'Z')? (C) - 'A' + 'a': (C))
 #endif
 
+/**
+ *
+ * @param cmd
+ */
 const ConsoleCommand * find_command(const char * cmd) {
   ConsoleCommand ** asp_p = NULL, dummy;
   ConsoleCommand * dummy_p;
@@ -836,25 +994,23 @@ const ConsoleCommand * find_command(const char * cmd) {
   return asp;
 }
 
-
 /**
- * Returns a pointer to the head of an array of ConsoleCommands
- * sorted by category, then by name.
- *
- * It's num_commands long.
+ * Returns a pointer to the head of an array of ConsoleCommands sorted by
+ * category, then by name.  It's num_commands long.
  */
 ConsoleCommand ** get_cat_sorted_commands(void) {
     return cat_sorted_commands;
 }
 
-
-/* Tries to handle command cp (with optional params in cpnext, which may be null)
- * as a local command. If this was a local command, returns true to indicate
- * command was handled.
- * This code was moved from extended_command so scripts ca issue local commands
- * to handle keybindings or anything else.
+/**
+ * Tries to handle command cp (with optional params in cpnext, which may be
+ * null) as a local command. If this was a local command, returns true to
+ * indicate command was handled.  This code was moved from extended_command so
+ * scripts ca issue local commands to handle keybindings or anything else.
+ *
+ * @param cp
+ * @param cpnext
  */
-
 int handle_local_command(const char* cp, const char * cpnext) {
     const ConsoleCommand * cc = NULL;
 
@@ -868,7 +1024,7 @@ int handle_local_command(const char* cp, const char * cpnext) {
         char buf[MAX_BUF];
 
         snprintf(buf, MAX_BUF - 1, "Client command %s has no implementation!", cc->name);
-	draw_info(buf, NDI_RED);
+	draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, buf);
 
 	return FALSE;
     }
@@ -878,15 +1034,17 @@ int handle_local_command(const char* cp, const char * cpnext) {
     return TRUE;
 }
 
-/* This is an extended command (ie, 'who, 'whatever, etc).  In general,
- * we just send the command to the server, but there are a few that
- * we care about (bind, unbind)
+/**
+ * This is an extended command (ie, 'who, 'whatever, etc).  In general, we
+ * just send the command to the server, but there are a few that we care about
+ * (bind, unbind)
  *
- * The command passed to us can not be modified - if it is a keybinding,
- * we get passed the string that is that binding - modifying it effectively
+ * The command passed to us can not be modified - if it is a keybinding, we
+ * get passed the string that is that binding - modifying it effectively
  * changes the binding.
+ *
+ * @param ocommand
  */
-
 void extended_command(const char *ocommand) {
     const char *cp = ocommand;
     char *cpnext, command[MAX_BUF];
@@ -930,7 +1088,6 @@ void extended_command(const char *ocommand) {
     }
 }
 
-
 /* ------------------------------------------------------------------ */
 
 /* This list is used for the 'tab' completion, and nothing else.
@@ -964,11 +1121,13 @@ static const char *const commands[] = {
 };
 #define NUM_COMMANDS ((int)(sizeof(commands) / sizeof(char*)))
 
-/* Player has entered 'command' and hit tab to complete it.
- * See if we can find a completion.  Returns matching
- * command. Returns NULL if no command matches.
+/**
+ * Player has entered 'command' and hit tab to complete it.  See if we can
+ * find a completion.  Returns matching command. Returns NULL if no command
+ * matches.
+ *
+ * @param command
  */
-
 const char * complete_command(const char *command)
 {
     int i, len, display;
@@ -1027,10 +1186,12 @@ const char * complete_command(const char *command)
     if (match == NULL) {
         if (display) {
             strncat(list, "\n", 499 - strlen(list));
-            draw_info(list, NDI_BLACK);
+            draw_ext_info(
+                NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, list);
         }
         else
-            draw_info("No matching command.\n", NDI_BLACK);
+            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+                "No matching command.\n");
         /* No match. */
         return NULL;
     }
