@@ -1,9 +1,9 @@
 const char * const rcsid_gtk2_image_c =
-    "$Id$";
+    "$Id: image.c 12884 2010-04-12 06:58:52Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
-    Copyright (C) 2005 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2005-2008,2010 Mark Wedel & Crossfire Development Team
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -75,6 +75,9 @@ PixmapInfo *pixmaps[MAXPIXMAPNUM];
 
 int last_face_num=0;
 
+/* Do we have new images to display? */
+int have_new_image=0;
+
 /*
  * this is used to rescale big images that will be drawn in the inventory/look
  * lists.  What the code further below basically does is figure out how big the
@@ -301,8 +304,26 @@ int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *r
         return 1;
 
     if (pixmaps[pixmap_num] != pixmaps[0]) {
-        free_pixmap(pixmaps[pixmap_num]);
-        free(pixmaps[pixmap_num]);
+        /* As per bug 2938906, one can see image corruption when switching between
+         * servers.  The cause is that the cache table stores away
+         * a pointer to the pixmap[] entry - if we go and free it,
+         * the cache table can point to garbage, so don't free it.
+         * This causes some memory leak, but if/when there is good
+         * cache support for multiple servers, eventually the amount
+         * of memory consumed will reach a limit (it has every image of
+         * every server in memory
+         *
+         * The cause of image corruption requires a few different things:
+         * 1) images of the same name have different numbers on the 2 serves.
+         * 2) the image number is higher on the first than second server
+         * 3) the image using the high number does not exist/is different
+         *    on the second server, causing this routine to be called.
+         */
+
+         if (!use_config[CONFIG_CACHE]) {
+             free_pixmap(pixmaps[pixmap_num]);
+             free(pixmaps[pixmap_num]);
+         }
         pixmaps[pixmap_num] = pixmaps[0];
     }
 
@@ -390,6 +411,9 @@ int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *r
         ce->image_data = pi;
     }
     pixmaps[pixmap_num] = pi;
+    if (use_config[CONFIG_CACHE])
+        have_new_image++;
+
     return 0;
 }
 
