@@ -1,5 +1,5 @@
-static char *rcsid_cfsndserv_c =
-    "$Id$";
+static char *rcsid_sound_src_cfsndserv_c =
+    "$Id: cfsndserv.c 12153 2009-08-20 23:31:20Z kbulgrien $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -23,22 +23,20 @@ static char *rcsid_cfsndserv_c =
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* Comment from the original author is below.  In addition to OSS
- * and ALSA, sun sound is also supported.
- */
-
-/*
+/**
+ * @file sound-src/cfsndserv.c
+ * The server for sound support for the client.  In addition to OSS and ALSA,
+ * sun sound is also supported.  Comment from the original author is below.
+ *
  * (c) 1998 Jacek Konieczny <jajcus@zeus.polsl.gliwice.pl>
  *
- * This file contains the server for sound support for the client.
- * It supports both ALSA_SOUND and OSS_SOUND. Any other sound system support
- * can be easily added - only new init_audio and audio_play
- * need be written.
+ * This file contains the server for sound support for the client.  It
+ * supports both ALSA_SOUND and OSS_SOUND. Any other sound system support can
+ * be easily added - only new init_audio and audio_play need be written.
  *
  * If you have any problems please e-mail me.
  */
 
-
 /*#define ALSA_SOUND*/
 /*#define OSS_SOUND*/
 /*#define SGI_SOUND*/
@@ -79,7 +77,7 @@ static char *rcsid_cfsndserv_c =
 #include <ctype.h>
 #include <errno.h>
 
-#include "newclient.h"
+#include "shared/newclient.h"
 #include "soundsdef.h"
 
 
@@ -951,7 +949,7 @@ int main(int argc, char *argv[])
     int inbuf_pos=0,sndbuf_pos=0;
     fd_set inset,outset;
 
-    printf ("%s\n",rcsid_cfsndserv_c);
+    printf ("%s\n", rcsid_sound_src_cfsndserv_c);
     fflush(stdout);
     if (read_settings()) write_settings();
     if (init_sounds()) return 1;
